module ColorNet (module ColorNet) where

import Net
import Relation
import List


-------------------------------------------------------------------------------------------------------
-- Local classes and types
-------------------------------------------------------------------------------------------------------

class (Spanned p, Show p, Read p) => Places  p

class (Spanned a, Show a, Read a) => Actions a

class (Spanned c, Show c, Read c) => Colors  c

type Marks p c = Rel p c

type Trans a c = Rel a c



-------------------------------------------------------------------------------------------------------
-- Instance of Transition and State
--------------------------------------------------------------------------------------------------------

instance (Actions a, Colors c) => Transition (Trans a c)


instance (Places p, Colors c) => State (Marks p c) where
     splus  (Rel u) vv  = do (Rel v) <- vv ; ok (mrg junion u v) >>=  return . Rel
     sminus (Rel u) vv  = do (Rel v) <- vv ; ok (mrg jdiff  u v) >>=  return . Rel 

-------------------------------------------------------------------------------------------------------
-- Instance of Net
------------------------------------------------------------------------------------------------------

type  Incidence p a c d = a -> d -> Marks p c

data (Places p, Actions a, Colors c, Colors d, State s, Transition t) => ColorNet p a c d s t =
      ColorNet { mF :: Incidence p a c d, mB :: Incidence p a c d}


instance (Places p, Actions a, Colors c, Colors d)  =>
    Net (ColorNet p a c d) (Marks p c) (Trans a d) where
       fwrd = acc . mF
       bwrd = acc . mB 

-- auxiliar functions

mrg :: (a -> b -> c) -> (p -> a) -> (p -> b) -> p -> c
mrg f u v = \p -> f (u p) (v p)

ok :: Spanned p => (p -> [c]) -> [p -> c]
ok h = mapM_ h s_span >>  [head . h]

acc f t = let h (a,c) u = (f a c) `splus` u in foldr h [bot]  (relToList t)
-- acc f = foldr (splus . (uncurry f)) [bot] . relToList

