module DinningPhilosophers (
--		module ColorNet,
--		module TransSystem,
--		module ST,
--		module Relation,
		module DinningPhilosophers
	       ) where

import ColorNet
import TransSystem
import ST
import Relation

---------------------------------------------------------------------------------------------------
-- Exemple of the N dinning philosophers
--------------------------------------------------------------------------------------------------

data Pos = Eating | Thinking | Available
	   deriving (Eq, Show, Read)

data Act = Start | End
	 deriving (Eq, Show, Read)


instance Spanned Pos where
      s_span   = [ Eating, Thinking, Available ]

instance Spanned Act where
      s_span = [ Start, End ]



data Token = Cor Int   deriving (Eq, Show, Read)
instance Spanned Token where
      s_span = map Cor [0..4]


instance Places  Pos
instance Actions Act
instance Colors  Token


myF, myB :: Incidence Pos Act Token Token


neighb i = [i , (i+1) `mod` 5]

myF a (Cor i) = case a of {
          Start  -> Rel (\p -> case p of { Eating   -> []            ;
                                             Thinking -> [Cor i]       ;
                                             Available -> map Cor (neighb i) })  ;
          End    -> Rel (\p -> case p of { Eating   -> [Cor i]  ;
                                             Thinking -> []  ;
                                             Available -> [] })
	  }
					     

myB a (Cor i) = case a of {
          Start  -> Rel (\p -> case p of { Eating   -> [Cor i]  ;
                                             Thinking -> []  ;
                                             Available -> [] })  ;
          End    -> Rel (\p -> case p of { Eating   -> []  ;
                                             Thinking -> [Cor i]  ;
                                             Available ->  map Cor (neighb i)  })
	  }
	

dinPhil :: ColorNet Pos Act Token Token  (Rel Pos Token) (Trans Act Token)
dinPhil = ColorNet myF myB



inicio :: Rel Pos Token
inicio = Rel inicio_ where
     inicio_ p = case p of {
			   Eating    -> []     ;
			   Thinking  -> s_span ;
			   Available -> s_span
			   }




--- Testes

explode_   s     = explode dinPhil `on` s
accessibles_  s  = accessibles dinPhil `on` s

-- com IO

go :: (Read t, Show a) => (t -> ST [] s a) -> String -> s -> IO () 
go f s q = do t <- readIO s ; print (f t `on` q)

transition_, path_, choices_ :: String -> Marks Pos Token -> IO ()
transition_ = go (transition dinPhil)
path_       = go (path dinPhil)
choices_    = go (choices dinPhil)



-- alguns conjuntos de transies para testes

transicoes_simples, transicoes_duplas :: [Trans Act Token]

transicoes_simples = [listToRel [(Start, Cor i)] | i <- [0..4]]

transicoes_duplas = [listToRel [(Start, Cor i),(Start, Cor j)] | i <- [0..4] , j <- [i+1..4]]

